<?php
/**
 * Handles plugin table caching.
 *
 * @since 3.0.0
 * @package SWPTLSPRO
 */

namespace SWPTLSPRO;

// If direct access than exit the file.
defined( 'ABSPATH' ) || exit;
/**
 * Manages plugin Cache.
 *
 * @since 2.12.15
 * @package SWPTLSPRO
 */
class Cache {
	/**
	 * Cache expiration times.
	 */
	const TIMESTAMP_CACHE_TTL = 300; // 5 minutes for timestamp cache
	const DATA_CACHE_TTL = 2592000; // 30 days for data cache by default
	const UPDATE_CHECK_FREQUENCY = 300; // 5 minutes between update checks
	
	/**
	 * Retrieves the last updated timestamp of a Google Sheet using a Google Apps Script web app.
	 *
	 * @param string $sheet_id The ID of the Google Sheet.
	 *
	 * @return string|false The last updated timestamp if available, or false on failure.
	 */
	public function get_last_sheet_updated_timestamp( string $sheet_id ) {

		$url = 'https://script.google.com/macros/s/AKfycbz2lB2sri2hRmzITLtvtP0BdE-mYjXiz1WyGEeqpsEA_evkc6M8vxh7qHbeuDheDtXlAA/exec';
		$args = [
			'timeout' => 10,
			'headers' => [
				'Content-Type' => 'application/json',
			],
			'body'    => [
				'sheetID' => $sheet_id,
				'action'  => 'lastUpdatedTimestamp',
			],
		];
		$response = wp_remote_get( $url, $args );
		if ( is_wp_error( $response ) ) {
			return false;
		}

		$response_code = wp_remote_retrieve_response_code( $response );
		$body          = json_decode( wp_remote_retrieve_body( $response ) );

		if ( 200 !== $response_code || ! isset( $body->lastUpdatedTimestamp ) ) {
			return false;
		}
		return $body->lastUpdatedTimestamp;
	}


	/**
	 * Retrieves the last updated timestamp of a Google Sheet using a Google Apps Script web app.
	 *
	 * @param string $url The URL of the Google Sheet.
	 *
	 * @return string|false The last updated timestamp if available, or false on failure.
	 */
	public function get_last_sheet_updated_time( string $url ): string {
		$sheet_id     = swptls()->helpers->get_sheet_id( $url );
		$updated_time = $this->get_last_sheet_updated_timestamp( $sheet_id );
		if ( ! $updated_time ) {
			return false;
		}
		return strtotime( $updated_time );
	}


	/**
	 * Sets the last updated time for a Google Sheet.
	 *
	 * @param int    $tableID The ID of the table.
	 * @param string $url     The URL of the Google Sheet.
	 *
	 * @return bool True on success, false on failure.
	 */
	public function set_last_updated_time(int $tableID, string $url): bool {
        // Validate inputs
        if (empty($url) || $tableID <= 0) {
            return false;
        }
        $sheet_id = swptls()->helpers->get_sheet_id($url);
        
        if (empty($sheet_id)) {
            return false;
        }
        // Get the last updated timestamp from Google.
        $lastUpdatedTimestampStr = $this->get_last_sheet_updated_timestamp($sheet_id);

        if ($lastUpdatedTimestampStr === false) {
            return false;
        }

        // Convert to timestamp.
        $lastUpdatedTimestamp = strtotime($lastUpdatedTimestampStr);

        // Prepare option key.
        $timestamp_key = sprintf('gswpts_sheet_updated_time_%d', $tableID);

        // Save the timestamp with explicit autoload setting.
        $save_result = update_option($timestamp_key, $lastUpdatedTimestamp, 'no');

        // Verify the saved timestamp.
        $verified_timestamp = get_option($timestamp_key);
        // error_log('Verified saved timestamp: ' . print_r($verified_timestamp, true));

        return $save_result;
    }

	/**
	 * Get the dynamic data cache expiration time.
	 *
	 * @return int
	 */
	public function get_data_cache_ttl() {
		$minutes = get_option('cache_timestamp', 30);
		$cache_ttl = !empty($minutes) ? (int) $minutes : 30;
		$cache_ttl = $cache_ttl * 60; // Convert minutes to seconds
		
		return $cache_ttl;
	}

	/**
	 * Debugging function to check the saved timestamp.
	 *
	 * @param int $tableID The ID of the table.
	 */
	public function debug_timestamp(int $tableID): void {
		global $wpdb;
        $timestamp_key = sprintf('gswpts_sheet_updated_time_%d', $tableID);
        
        // Retrieve the timestamp using different methods.
        $option_timestamp = get_option($timestamp_key);
        $option_timestamp_raw = get_option($timestamp_key, 'not_found');
  
        $db_value = $wpdb->get_var($wpdb->prepare(
            "SELECT option_value FROM $wpdb->options WHERE option_name = %s",
            $timestamp_key
        ));
    }


	/**
	 * Checks if the Google Sheet has been updated since the last saved timestamp.
	 *
	 * @param int    $tableID The ID of the table.
	 * @param string $url     The URL of the Google Sheet.
	 *
	 * @return bool True if updated, false otherwise.
	 */
	public function is_updated(int $tableID, string $url): bool {
        // Prepare option key for last saved timestamp.
        $option_key = sprintf('gswpts_sheet_updated_time_%d', $tableID);
        
        // Get the saved timestamp .
        $saved_timestamp = get_option($option_key);
        $sheet_id = swptls()->helpers->get_sheet_id($url);
        if (empty($sheet_id)) {
            return false;
        }

        // Get the current sheet update timestamp.
        $current_timestamp_str = $this->get_last_sheet_updated_timestamp($sheet_id);

        if ($current_timestamp_str === false) {
            return false;
        }

        // Convert to timestamp.
        $current_timestamp = strtotime($current_timestamp_str);

        // Ensure saved_timestamp is a valid number.
        $saved_timestamp = $saved_timestamp !== false ? intval($saved_timestamp) : 0;
        // Compare timestamps.
        $is_updated = $current_timestamp > $saved_timestamp;
    
        return $is_updated;
    }



	/**
	 * Generates a unique cache key for a given table ID and data type.
	 *
	 * @param int    $table_id Table identifier.
	 * @param string $type     Type of data (e.g., 'data', 'styles', 'images').
	 * @return string          Generated cache key.
	 */
	private function get_cache_key(int $table_id, string $type = 'data') {
		return sprintf('gswpts_sheet_%s_%d', $type, $table_id);
	}

	/**
	 * Saves data to the WordPress transient cache.
	 *
	 * @param int    $table_id Table identifier.
	 * @param mixed  $data     Data to be cached.
	 * @param string $type     Type of data (default: 'data').
	 * @return bool            True on success, false on failure.
	 */
	private function save_to_cache(int $table_id, $data, string $type = 'data') {
		if (empty($data) || $table_id <= 0) {
			return false;
		}

		// Set the cache expiration using a filter to allow customization.
		// $expiration = apply_filters('gswpts_cache_expiration', self::DATA_CACHE_TTL);
		$expiration = apply_filters('gswpts_cache_expiration', $this->get_data_cache_ttl());
		$key = $this->get_cache_key($table_id, $type);

		return set_transient($key, $data, $expiration);
	}

	/**
	 * Retrieves data from the WordPress transient cache.
	 *
	 * @param int    $table_id Table identifier.
	 * @param string $type     Type of data (default: 'data').
	 * @return mixed|null      Cached data or null if not found or invalid ID.
	 */
	private function get_from_cache(int $table_id, string $type = 'data') {
		if ($table_id <= 0) {
			return null;
		}

		$key = $this->get_cache_key($table_id, $type);
		return get_transient($key);
	}

	/**
	 * Caches sheet data for a given table.
	 *
	 * @param int   $tableID       Table ID.
	 * @param mixed $sheetResponse Sheet data to cache.
	 * @return bool                Result of the caching operation.
	 */
	public function saveSheetData(int $tableID, $sheetResponse) {
		return $this->save_to_cache($tableID, $sheetResponse, 'data');
	}

	/**
	 * Caches sheet style data for a given table.
	 *
	 * @param int   $tableID     Table ID.
	 * @param mixed $sheetStyles Style data to cache.
	 * @return bool              Result of the caching operation.
	 */
	public function saveSheetStyles(int $tableID, $sheetStyles) {
		return $this->save_to_cache($tableID, $sheetStyles, 'styles');
	}

	/**
	 * Caches merged style data (e.g., merged cells) for a given table.
	 *
	 * @param int   $tableID       Table ID.
	 * @param mixed $sheetMergedata Merged style data to cache.
	 * @return bool                Result of the caching operation.
	 */
	public function saveMergedStyles(int $tableID, $sheetMergedata) {
		return $this->save_to_cache($tableID, $sheetMergedata, 'merged');
	}

	/**
	 * Caches sheet image data for a given table.
	 *
	 * @param int   $tableID   Table ID.
	 * @param mixed $imagesData Image data to cache.
	 * @return bool            Result of the caching operation.
	 */
	public function saveSheetImages(int $tableID, $imagesData) {
		return $this->save_to_cache($tableID, $imagesData, 'images');
	}

	/**
	 * Caches sheet link data (e.g., hyperlinks) for a given table.
	 *
	 * @param int   $tableID Table ID.
	 * @param mixed $linkData Link data to cache.
	 * @return bool          Result of the caching operation.
	 */
	public function saveSheetLink(int $tableID, $linkData) {
		return $this->save_to_cache($tableID, $linkData, 'link');
	}

	/**
	 * Retrieves cached sheet data.
	 *
	 * @param int $table_id Table ID.
	 * @return mixed        Cached data or null.
	 */
	public function get_saved_sheet_data(int $table_id) {
		return $this->get_from_cache($table_id, 'data');
	}

	/**
	 * Retrieves cached sheet styles.
	 *
	 * @param int $table_id Table ID.
	 * @return mixed        Cached styles or null.
	 */
	public function getSavedSheetStyles(int $table_id) {
		return $this->get_from_cache($table_id, 'styles');
	}

	/**
	 * Retrieves cached merged style data.
	 *
	 * @param int $table_id Table ID.
	 * @return mixed        Cached merged data or null.
	 */
	public function getSavedMergeStyles(int $table_id) {
		return $this->get_from_cache($table_id, 'merged');
	}

	/**
	 * Retrieves cached sheet image data.
	 *
	 * @param int $table_id Table ID.
	 * @return mixed        Cached images or null.
	 */
	public function getSavedSheetImages(int $table_id) {
		return $this->get_from_cache($table_id, 'images');
	}

	/**
	 * Retrieves cached sheet link data.
	 *
	 * @param int $table_id Table ID.
	 * @return mixed        Cached link data or null.
	 */
	public function getSavedSheetLinkStyles(int $table_id) {
		return $this->get_from_cache($table_id, 'link');
	}

}